// FOPVisioStencilConvert.cpp: implementation of the CFOPVisioStencilConvert class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FOPVisioStencilConvert.h"
#include "FOPolygonShape.h"
#include "FOEllipseShape.h"
#include "FOLineShape.h"
#include "FORectShape.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

void CFOPConceptStencilConvert::Convert()
{
	ProcessXmlDocument();
}

void CFOPConceptStencilConvert::Dispose()
{
}

void CFOPConceptStencilConvert::ProcessXmlDocument()
{
	if (TRUE == m_fopXML.Load(m_strConceptFile))
	{
		CXDXmlNode xmlNode = m_fopXML.GetNodeList(_T("ConceptDrawDocument"));
		
		ProcessPages(xmlNode.GetNode(_T("Pages")));
	}
}

void CFOPConceptStencilConvert::ProcessPages(CXDXmlNode nodePages)
{
	if (false == nodePages.IsEmpty())
	{		
		CXDXmlNode nodePageList = nodePages.GetNodeList(_T("Page"));
		int nSize = nodePageList.GetNodeListSize();
		for (int i=0; i<nSize; i++)
		{
			ProcessPage(nodePageList.GetItemNode(i));
		}
	}
}

void CFOPConceptStencilConvert::ProcessPage(CXDXmlNode nodePage)
{
	if (false == nodePage.IsEmpty())
	{		
		Group *pParent = NULL;
		CFOPMasterInfo masterInfo;

		CXDXmlNode nodeShpesList = nodePage.GetNodeList(_T("Shapes"));
		int nSize = nodeShpesList.GetNodeListSize();
		for (int i=0; i<nSize; i++)
		{
			ProcessShapes(nodeShpesList.GetItemNode(i), pParent, masterInfo);
		}
	}
}

void CFOPConceptStencilConvert::ProcessShapes(CXDXmlNode node, Group *pParent, CFOPMasterInfo &masterInfo)
{
	if (false == node.IsEmpty())
	{
		int nSize = 0, i = 0;
		CXDXmlNode nodeTemp;

		nodeTemp = node.GetNodeList(_T("Shape"));
		nSize = nodeTemp.GetNodeListSize();
		for (i=0; i<nSize; i++)
		{
			ProcessShape(nodeTemp.GetItemNode(i), pParent, masterInfo);
		}
		
		nodeTemp = node.GetNodeList(_T("Shapes"));
		nSize = nodeTemp.GetNodeListSize();
		for (i=0; i<nSize; i++)
		{
			ProcessShapes(nodeTemp.GetItemNode(i), pParent, masterInfo);
		}
	}
}

void CFOPConceptStencilConvert::ProcessShape(CXDXmlNode nodeShape, Group *pParent, CFOPMasterInfo &masterInfo)
{
	pParent;
	if (true == nodeShape.IsEmpty())
		return;
	
	CXDXmlNode nodeShapes = nodeShape.GetNode(_T("Shapes"));
	if (false == nodeShapes.IsEmpty())
	{
		Group *pNewGroup = NULL;
		CFOPMasterInfo newMasterInfo;

		ProcessShapes(nodeShapes, pNewGroup, newMasterInfo);
	}

	CXDXmlNode nodeGeomsList =nodeShape.GetNodeList(_T("Geometries"));
	int nSize = nodeGeomsList.GetNodeListSize();
	for (int i=0; i<nSize; i++)
	{
		CXDXmlNode childNode = nodeGeomsList.GetItemNode(i);

		CXDXmlNode nodeGeomList = childNode.GetNodeList(_T("Geom"));
		int nGeomSize = nodeGeomList.GetNodeListSize();
		for (int j=0; j<nGeomSize; j++)
		{
			CXDXmlNode childGeomNode = nodeGeomList.GetItemNode(j);

			ProcessGeom(childGeomNode, masterInfo);
		}
	}
}

void CFOPConceptStencilConvert::ProcessGeom(CXDXmlNode nodeGeom, CFOPMasterInfo &masterInfo)
{
	if (false != nodeGeom.IsEmpty())
		return;

	FOPSimplePolygon simPath;
	FOPSimpleCompositePolygon curPath;

	CXDXmlNode childNodes = nodeGeom.GetChildNodes();
	int nSize = childNodes.GetNodeListSize();
	for (int i=0; i<nSize; i++)
	{
		CXDXmlNode childNode = childNodes.GetItemNode(i);
		CString strText = childNode.GetNodeName();

		if (strText == _T("Start"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
//			simPath.MoveFirstTo(FOPPoint(nX, nY));
			curPath.MoveTo(FOPPoint(nX, nY));
		}
		else if (strText == _T("LineTo"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
//			simPath.LineTo(FOPPoint(nX, nY));
			curPath.LineTo(FOPPoint(nX, nY));
		}
		else if (strText == _T("ArcTo"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			int nA = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("A")));
			int nB = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("B")));
//			simPath.QuadTo(FOPPoint(nA, nB), FOPPoint(nX, nY));
			curPath.ArcTo(FOPPoint(nA, nB), FOPPoint(nX, nY));
		}
		else if (strText == _T("EllipseTo"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			int nA = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("A")));
			int nB = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("B")));
			int nC = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("C")));
			int nD = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("D")));
			nX;nY;
			curPath.AddEllipse(FOPRect(nA, nB, nC, nD));
		}
		else if (strText == _T("SplineStart"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			int nA = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("A")));
			int nB = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("B")));
//			simPath.QuadTo(FOPPoint(nA, nB), FOPPoint(nX, nY));
			curPath.QuadTo(FOPPoint(nA, nB), FOPPoint(nX, nY));
		}
		else if (strText == _T("SplineKnot"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			int nA = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("A")));
			int nB = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("B")));
//			simPath.QuadTo(FOPPoint(nA, nB), FOPPoint(nX, nY));
			curPath.QuadTo(FOPPoint(nA, nB), FOPPoint(nX, nY));
		}
	}

	if (curPath.Count() > 0)
	{
		CFOPathShape *pReturn = new CFOPathShape;
		pReturn->AddRef();
		pReturn->Create(curPath);
//		pReturn->AddSubPath(simPath);
		pReturn->SetEditPointAllowed(TRUE);

		pReturn->SetObjectCaption(masterInfo.strName);
		pReturn->SetToolTip(masterInfo.strPrompt);
		pReturn->SetShowLabelAtFirst(TRUE);

		m_pModel->InsertShape(pReturn);
		pReturn->Release();
		pReturn = NULL;
	}
}

float CFOPConceptStencilConvert::GetRotationAngle(double dAngle)
{
	return (float)(dAngle*(180.0/3.14159265358979));			
}

int CFOPConceptStencilConvert::ConvertMM2Point_Horz(double dHorz)
{
//	m_pModel->ConvertToMapLog(FUNIT_MM, dHorz, FALSE);
	return int(dHorz);
}

int CFOPConceptStencilConvert::ConvertMM2Point_Vert(double dVert)
{
//	m_pModel->ConvertToMapLog(FUNIT_MM, dVert, TRUE);
	return int(dVert);
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

void CFOPVisioStencilConvert::CreateSymbolPalette()
{
	m_pPaletteTemp = new SymbolPalette;
	
	m_pPaletteTemp->m_strPaletteName = m_strSymbolPaletteName;
}

void CFOPVisioStencilConvert::Convert()
{
	InternalConvert();
}

void CFOPVisioStencilConvert::Dispose()
{
}

void CFOPVisioStencilConvert::InternalConvert()
{
	ProcessXmlDocument();
}

void CFOPVisioStencilConvert::ProcessXmlDocument()
{
	if (TRUE == m_fopXML.Load(m_strTempVsxFile))
	{
		CXDXmlNode xmlNode = m_fopXML.GetNodeList(_T("VisioDocument"));

		ProcessDocProps(xmlNode.GetNode(_T("DocumentProperties")));
//		ProcessProps(xmlNode.GetNode(_T("DocumentSettings")));
		ProcessColors(xmlNode.GetNode(_T("Colors")));
		ProcessFaceNames(xmlNode.GetNode(_T("FaceNames")));
		ProcessStyleSheets(xmlNode.GetNode(_T("StyleSheets")));
//		ProcessProps(xmlNode.GetNode(_T("DocumentSheet")));
		ProcessMasters(xmlNode.GetNode(_T("Masters")));
// 		ProcessProps(xmlNode.GetNode(_T("Windows")));
// 		ProcessProps(xmlNode.GetNode(_T("EventList")));
	}
}

void CFOPVisioStencilConvert::ProcessFaceNames(CXDXmlNode nodeFaceNames)
{
	m_hashFaceNames.RemoveAll();
	if (true == nodeFaceNames.IsEmpty())
		return;
	
	CXDXmlNode nodeFaceName = nodeFaceNames.GetNodeList(_T("FaceName"));
	int nSize = nodeFaceName.GetNodeListSize();
	for (int i=0; i<nSize; i++)
	{
		CXDXmlNode nodeChind = nodeFaceName.GetItemNode(i);
		if (false == nodeChind.IsEmpty())
		{
			CXDNamedNode nameNode = nodeChind.GetNamedNode();
			int nID = nameNode.GetNodeInt(_T("ID"));;
			CString strName = nameNode.GetNodeText(_T("Name"));
			if (nID > 0 && strName != _T("") && FALSE == strName.IsEmpty())
				m_hashFaceNames.SetAt(nID, strName);
		}
	}
}

void CFOPVisioStencilConvert::ProcessColors(CXDXmlNode nodeColors)
{
	m_hashColors.RemoveAll();
	if (true == nodeColors.IsEmpty())
		return;

	CXDXmlNode nodeColorEntry = nodeColors.GetNodeList(_T("ColorEntry"));
	int nSize = nodeColorEntry.GetNodeListSize();
	for (int i=0; i<nSize; i++)
	{
		CXDXmlNode nodeChind = nodeColorEntry.GetItemNode(i);
		if (false == nodeChind.IsEmpty())
		{
			CXDNamedNode nameNode = nodeChind.GetNamedNode();
			int nID = nameNode.GetNodeInt(_T("IX"));
			// RGB="#1A1A1A"
			COLORREF crRGB = ProcessColor(nameNode.GetNodeText(_T("RGB")), false);
			m_hashColors.SetAt(nID, crRGB);
		}
	}
}

void CFOPVisioStencilConvert::ProcessStyleSheets(CXDXmlNode nodeStyleSheets)
{
	if (true == nodeStyleSheets.IsEmpty())
		return;

	CXDXmlNode nodeStyleSheet = nodeStyleSheets.GetNodeList(_T("StyleSheet"));
	int nSize = nodeStyleSheet.GetNodeListSize();
	for (int i=0; i<nSize; i++)
	{
		CXDXmlNode nodeChind = nodeStyleSheet.GetItemNode(i);
		ProcessStyleSheet(nodeChind);
	}
}

void CFOPVisioStencilConvert::ProcessStyleSheet(CXDXmlNode nodeStyleSheet)
{
	if (false == nodeStyleSheet.IsEmpty())
	{
		CXDNamedNode nameNode = nodeStyleSheet.GetNamedNode();
		int nID = nameNode.GetNodeInt(_T("ID"));

		HelperFillStyle *pFillStyle = new HelperFillStyle;
		HelperLineStyle *pLineStyle = new HelperLineStyle;
		HelperTextStyle *pTextStyle = new HelperTextStyle;
		HelperCharStyle *pCharStyle = new HelperCharStyle;	
		HelperParaStyle *pParaStyle = new HelperParaStyle;

		m_hashFillStyle.SetAt(nID, pFillStyle);
		m_hashLineStyle.SetAt(nID, pLineStyle);
		m_hashTextStyle.SetAt(nID, pTextStyle);
		m_hashCharStyle.SetAt(nID, pCharStyle);
		m_hashParaStyle.SetAt(nID, pParaStyle);

		pFillStyle->m_nInheritedFrom = nameNode.GetNodeInt(_T("FillStyle"));
		pLineStyle->m_nInheritedFrom = nameNode.GetNodeInt(_T("LineStyle"));
		pTextStyle->m_nInheritedFrom = nameNode.GetNodeInt(_T("TextStyle"));
		pCharStyle->m_nInheritedFrom = nameNode.GetNodeInt(_T("TextStyle"));
		pParaStyle->m_nInheritedFrom = nameNode.GetNodeInt(_T("TextStyle"));

		ProcessFillNode(nodeStyleSheet.GetNode(_T("Fill")),	pFillStyle);
		ProcessLineNode(nodeStyleSheet.GetNode(_T("Line")),	pLineStyle);
		ProcessTextNode(nodeStyleSheet.GetNode(_T("TextBlock")),	pTextStyle);
		ProcessCharNode(nodeStyleSheet.GetNode(_T("Char")),	pCharStyle);
		ProcessParaNode(nodeStyleSheet.GetNode(_T("Para")),	pParaStyle);
	}
}

void CFOPVisioStencilConvert::ProcessParaNode(CXDXmlNode nodePara, HelperParaStyle *hlpParaStyle)
{
	if (false == nodePara.IsEmpty())
	{
		hlpParaStyle->m_fIndFirst	= nodePara.GetNodeFloat(_T("IndFirst"));
		hlpParaStyle->m_fIndLeft	= nodePara.GetNodeFloat(_T("IndLeft"));
		hlpParaStyle->m_fIndRight	= nodePara.GetNodeFloat(_T("IndRight"));
		hlpParaStyle->m_fSpLine		= nodePara.GetNodeFloat(_T("SpLine"));
		hlpParaStyle->m_fSpBefore	= nodePara.GetNodeFloat(_T("SpBefore"));
		hlpParaStyle->m_fSpAfter	= nodePara.GetNodeFloat(_T("SpAfter"));
		hlpParaStyle->m_nHorzAlign	= nodePara.GetNodeInt(_T("HorzAlign"));
		hlpParaStyle->m_fTextPosAfterBullet = nodePara.GetNodeFloat(_T("TextPosAfterBullet"));
	}
}

COLORREF ConvertToColor(CString str)
{
	return (COLORREF)_ttoi(str);
}

void CFOPVisioStencilConvert::ProcessTextNode(CXDXmlNode nodeText, HelperTextStyle *hlpTextStyle)
{
	if (false == nodeText.IsEmpty())
	{
		hlpTextStyle->m_clrTextBkgnd = ConvertToColor(nodeText.GetNodeText(_T("TextBkgnd")));
	}
}

void CFOPVisioStencilConvert::ProcessCharNode(CXDXmlNode nodeChar, HelperCharStyle *hlpCharStyle)
{
	if (false == nodeChar.IsEmpty())
	{
		hlpCharStyle->m_nFontID	= nodeChar.GetNodeInt(_T("Font"));
		hlpCharStyle->m_clrTextColor= ConvertToColor(nodeChar.GetNodeText(_T("Color")));
		hlpCharStyle->m_fFontSize	= nodeChar.GetNodeFloat(_T("Size"));
		hlpCharStyle->m_nLangID	= nodeChar.GetNodeInt(_T("Langid"));

		hlpCharStyle->m_enumStyle	= nodeChar.GetNodeText(_T("Style"));

		hlpCharStyle->m_nCase	= nodeChar.GetNodeInt(_T("Case"));
		hlpCharStyle->m_nRTL	= nodeChar.GetNodeInt(_T("RTLText"));
		hlpCharStyle->m_nStrikeThrough	= nodeChar.GetNodeInt(_T("Strikethru"));
		hlpCharStyle->m_nDoubleUnderline= nodeChar.GetNodeInt(_T("DblUnderline"));
		hlpCharStyle->m_nVertical	= nodeChar.GetNodeInt(_T("UseVertical"));
		hlpCharStyle->m_nPosition	= nodeChar.GetNodeInt(_T("Pos"));
	}
}

void CFOPVisioStencilConvert::ProcessLineNode(CXDXmlNode nodeLine, HelperLineStyle *hlpLineStyle)
{
	if (false == nodeLine.IsEmpty())
	{
		hlpLineStyle->m_bNoLine = nodeLine.GetNodeBool(_T("Case"));
		hlpLineStyle->m_clrLineColor= ConvertToColor(nodeLine.GetNodeText(_T("LineColor")));
		hlpLineStyle->m_fLineWeight	= nodeLine.GetNodeFloat(_T("LineWeight"));
		hlpLineStyle->m_nLinePattern= nodeLine.GetNodeInt(_T("LinePattern"));
		hlpLineStyle->m_nLineCap	= nodeLine.GetNodeInt(_T("LineCap"));
	}
}

void CFOPVisioStencilConvert::ProcessFillNode(CXDXmlNode nodeFill, HelperFillStyle *hlpFillStyle)
{
	if (false == nodeFill.IsEmpty())
	{
		hlpFillStyle->m_clrFillBkgnd = ConvertToColor(nodeFill.GetNodeText(_T("FillBkgnd")));
		hlpFillStyle->m_clrFillForegnd = ConvertToColor(nodeFill.GetNodeText(_T("FillForegnd")));

		hlpFillStyle->m_nFillPattern = nodeFill.GetNodeInt(_T("FillPattern"));

		hlpFillStyle->m_fFillForegndTransparency = nodeFill.GetNodeFloat(_T("FillForegndTrans"));
		hlpFillStyle->m_fFillBkgndTransparency = nodeFill.GetNodeFloat(_T("FillBkgndTrans"));
		
		hlpFillStyle->m_clrShdwBkgnd = ConvertToColor(nodeFill.GetNodeText(_T("ShdwBkgnd")));
		hlpFillStyle->m_clrShdwForegnd = ConvertToColor(nodeFill.GetNodeText(_T("ShdwForegnd")));

		hlpFillStyle->m_nShdwPattern = nodeFill.GetNodeInt(_T("ShdwPattern"));

		hlpFillStyle->m_nShdwOffsetX = nodeFill.GetNodeInt(_T("ShapeShdwOffsetX"));
		hlpFillStyle->m_nShdwOffsetY = nodeFill.GetNodeInt(_T("ShapeShdwOffsetY"));
	}
}

void CFOPVisioStencilConvert::ProcessDocProps(CXDXmlNode nodeDocProps)
{
	if (nodeDocProps.IsEmpty() == false)
	{
		m_strSymbolPaletteName = nodeDocProps.GetNodeText(_T("Title"));
	}
}

void CFOPVisioStencilConvert::ProcessMasters(CXDXmlNode nodeMasters)
{
	if (false == nodeMasters.IsEmpty())
	{
		CreateSymbolPalette();

		CXDXmlNode nodeMaster = nodeMasters.GetNodeList(_T("Master"));
		int nSize = nodeMaster.GetNodeListSize();
		for (int i=0; i<nSize; i++)
		{
			ProcessMaster(nodeMaster.GetItemNode(i));
		}
	}
}

void CFOPVisioStencilConvert::ProcessMaster(CXDXmlNode nodeMaster)
{
	CXDNamedNode nodeName = nodeMaster.GetNamedNode();

	CFOPMasterInfo masterInfo;
	masterInfo.strName = nodeName.GetNodeText(_T("Name"));
	masterInfo.strPrompt = nodeName.GetNodeText(_T("Prompt"));

	CXDXmlNode nodeShapes = nodeMaster.GetNodeList(_T("Shapes"));
	int nSize = nodeShapes.GetNodeListSize();
	for (int i=0; i<nSize; i++)
	{
		CXDXmlNode nodeShape = nodeShapes.GetItemNode(i);
		
		m_bMainShape = true;

		ProcessShapes(nodeShape, m_pSymbolTemp, masterInfo);

//		int start;
//		m_pSymbolTemp->AppendChildren( mainShapeChildren, out start);
//		m_pSymbolTemp->m_strName = strSymbolName;
		
		// update EditStyle
		EditStyle *pStlEdit = new EditStyle;
		
		// find shape node
		CXDXmlNode nodeShapeList = nodeShapes.GetNodeList(_T("Shape"));
		int nCount = nodeShapeList.GetNodeListSize();
		for (int j=0; j<nCount; j++)
		{
			CXDXmlNode node = nodeShapeList.GetItemNode(j);
			ProcessProtection(nodeShapeList.GetNode(_T("Protection")), pStlEdit);
		}

		if (pStlEdit != NULL)
		{
			// update EditStyle
// 			m_pSymbolTemp->m_nEditStyle.SetAllowChangeWidth(pStlEdit->GetAllowChangeWidth());
// 			m_pSymbolTemp->m_nEditStyle.SetAllowChangeHeight(pStlEdit->GetAllowChangeHeight());
// 			m_pSymbolTemp->m_nEditStyle.SetAllowMoveX(pStlEdit->GetAllowMoveX());
// 			m_pSymbolTemp->m_nEditStyle.SetAllowMoveY(pStlEdit->GetAllowMoveY());
// 			m_pSymbolTemp->m_nEditStyle.SetAllowRotate(pStlEdit->GetAllowRotate());
// 			m_pSymbolTemp->m_nEditStyle.SetAllowSelect(pStlEdit->GetAllowSelect());
// 			m_pSymbolTemp->m_nEditStyle.SetAllowDelete(pStlEdit->GetAllowDelete());
// 			m_pSymbolTemp->m_nEditStyle.SetAspectRatio(pStlEdit->GetAspectRatio());
// 			m_pSymbolTemp->m_nEditStyle.SetAllowVertexEdit(pStlEdit->GetAllowVertexEdit());
		}

//		m_paletteTemp.AppendChild( m_pSymbolTemp);
		break;
	}
}

void CFOPVisioStencilConvert::ProcessShapes(CXDXmlNode node, Group *pParent, CFOPMasterInfo &masterInfo)
{
	if (false == node.IsEmpty())
	{
		int nSize = 0, i = 0;
		CXDXmlNode nodeTemp;

		nodeTemp = node.GetNodeList(_T("Shape"));
		nSize = nodeTemp.GetNodeListSize();
		for (i=0; i<nSize; i++)
		{
			ProcessShape(nodeTemp.GetItemNode(i), pParent, masterInfo);
		}
		
		nodeTemp = node.GetNodeList(_T("Shapes"));
		nSize = nodeTemp.GetNodeListSize();
		for (i=0; i<nSize; i++)
		{
			ProcessShapes(nodeTemp.GetItemNode(i), pParent, masterInfo);
		}
	}
}

void CFOPVisioStencilConvert::ProcessShape(CXDXmlNode nodeShape, Group *pParent, CFOPMasterInfo &masterInfo)
{
	if (true == nodeShape.IsEmpty())
		return;
	
	CArrayXDXmlNode lstConnections;
	
//////////////////////////////////////////////////////////////////////////
	CXDNamedNode nodeName = nodeShape.GetNamedNode();
	CString strUnique = nodeName.GetNodeText(_T("UniqueID"));

	CXDXmlNode nodeShapes = nodeShape.GetNode(_T("Shapes"));
	if (false == nodeShapes.IsEmpty())
	{
		Group *pGroup = NULL;
	
		if (false == m_bTopShape)
		{
			pParent = new Group;
			pGroup = pParent;
			CXDNamedNode nameTemp = nodeShapes.GetNamedNode();
			pGroup->m_strName = nameTemp.GetNodeText(_T("NameU"));
			if (pGroup->m_strName == _T(""))
				pGroup->m_strName = _T("Shape") + nameTemp.GetNodeText(_T("ID"));
		}
		
		XForm xf;
		EditStyle *pEitStyle = new EditStyle;
		
		CXDXmlNode nodeProtection =nodeShapes.GetNode(_T("Protection"));
		ProcessProtection(nodeProtection, pEitStyle);

		if (!pGroup->Equals(*pParent))
		{
			CXDXmlNode nodeProtection =nodeShapes.GetNode(_T("XForm"));
			if (nodeProtection.IsEmpty() == false)
				ProcessXForm(nodeProtection, xf);
			
			pGroup->m_szSize = CSize((int)xf.m_fWidth, (int)xf.m_fHeight);
			pGroup->m_ptPinPoint = CPoint((int)xf.m_fPinX, (int)pParent->m_szSize.cy - (int)xf.m_fPinY);
			pGroup->m_szPinPointOffset = CSize((int)xf.m_fLocPinX, (int)xf.m_fHeight - (int)xf.m_fLocPinY);
			pGroup->m_fRotationAngle = ( xf.m_bFlipY) ? xf.m_fRotationAngle : -xf.m_fRotationAngle;
			pGroup->m_bFlipX = xf.m_bFlipX;
			pGroup->m_bFlipY = xf.m_bFlipY;
		}
		
	//	Real for Process Shapes
		ProcessShapes(nodeShapes, pGroup, masterInfo);
		
	//	int start
		if( !pGroup->Equals(*pParent))
		{
		//	pGroup->AppendChildren( grchildren, out start);
			
			pGroup->m_ptPinPoint = CPoint((int)xf.m_fPinX, (int)pParent->m_szSize.cy - (int)xf.m_fPinY);
			pGroup->m_szPinPointOffset = CSize((int)xf.m_fLocPinX, (int)xf.m_fHeight - (int)xf.m_fLocPinY);
			pGroup->m_fRotationAngle = ( xf.m_bFlipY) ? xf.m_fRotationAngle : -xf.m_fRotationAngle;
			pGroup->m_bFlipX = xf.m_bFlipX;
			pGroup->m_bFlipY = xf.m_bFlipY;
			
		//	update EditStyle
			pGroup->m_nEditStyle.SetAllowChangeWidth(pEitStyle->GetAllowChangeWidth());
			pGroup->m_nEditStyle.SetAllowChangeHeight(pEitStyle->GetAllowChangeHeight());
			pGroup->m_nEditStyle.SetAllowMoveX(pEitStyle->GetAllowMoveX());
			pGroup->m_nEditStyle.SetAllowMoveY(pEitStyle->GetAllowMoveY());
			pGroup->m_nEditStyle.SetAllowRotate(pEitStyle->GetAllowRotate());
			pGroup->m_nEditStyle.SetAllowSelect(pEitStyle->GetAllowSelect());
			pGroup->m_nEditStyle.SetAllowDelete(pEitStyle->GetAllowDelete());
			pGroup->m_nEditStyle.SetAspectRatio(pEitStyle->GetAspectRatio());
			pGroup->m_nEditStyle.SetAllowVertexEdit(pEitStyle->GetAllowVertexEdit());
			
		//	children.Add( group);
		}
	}

	EditStyle *pStyleEdit = new EditStyle;
	CXDXmlNode nodeProtection =nodeShape.GetNode(_T("Protection"));
	ProcessProtection(nodeProtection, pStyleEdit);
	
	XForm xfShapeProp, xfTextProp;
	CXDXmlNode nodeXForm =nodeShape.GetNode(_T("XForm"));
	if (nodeXForm.IsEmpty() == false)
	{
		ProcessXForm(nodeXForm, xfShapeProp);
		if ( m_bMainShape)
		{
		// 	fYCenter = xfShapeProp.m_fPinY + xfShapeProp.m_fHeight;
		// 	fXCenter = xfShapeProp.m_fPinX;
		// 	pParent->m_szSize = CSize((int)xfShapeProp.m_fPinX + (int)xfShapeProp.m_fWidth, (int)xfShapeProp.m_fHeight + (int)xfShapeProp.m_fPinY);
		// 	pParent->m_ptPinPoint = CPoint((int)xfShapeProp.m_fPinX, (int)pParent->m_szSize.cy - (int)xfShapeProp.m_fPinY);
		// 	pParent->m_szPinPointOffset = CSize((int)xfShapeProp.m_fLocPinX, (int)xfShapeProp.m_fHeight - (int)xfShapeProp.m_fLocPinY);
		// 	pParent->m_fRotationAngle = (xfShapeProp.m_bFlipY)?xfShapeProp.m_fRotationAngle:-xfShapeProp.m_fRotationAngle;
		// 	pParent->m_bFlipX = xfShapeProp.m_bFlipX;
		// 	pParent->m_bFlipY = xfShapeProp.m_bFlipY;
		// 	m_bMainShape = false;
		}

		// ( stencil with text only has no TextXForm node defining text block properties)
		// so we have to take shape (XForm) properties
		xfTextProp = xfShapeProp;
	}
	
	CXDXmlNode nodeTextXForm =nodeShape.GetNode(_T("TextXForm"));
	ProcessTextXForm(nodeTextXForm, xfTextProp);
	
	int nGeomCount = -1;
	int nGeomsShown = -1;
	CXDXmlNode nodeGeom =nodeShape.GetNode(_T("Geom"));
	if (false == nodeGeom.IsEmpty())
	{
		bool bNoShow = (1 == nodeGeom.GetNodeInt(_T("NoShow")));

		if (false == bNoShow ) nGeomsShown++;
		if (nGeomsShown < 1 && false == bNoShow)
		{
			CString strHashKey = _T("");
			strHashKey.Format(_T("%s-%d"), strUnique, nGeomCount);

			ProcessGeom(nodeGeom, masterInfo);
		}
	}


	ProcessTextXForm(nodeTextXForm, xfTextProp);
	
	CXDXmlNode nodeText =nodeShape.GetNode(_T("Text"));
	ProcessText(nodeText, xfTextProp, xfShapeProp);
	
	CXDXmlNode nodeData =nodeShape.GetNode(_T("Foreigndata"));
	if (nodeData.IsEmpty() == false)
	{
		// if foreign type is Bitmap or Metafile ->
		// create appropriate Image and append it to fii.Image
		if (HasBitmapOrMetafile(nodeData))
		{
		}
	}
}

void CFOPVisioStencilConvert::ProcessGeom(CXDXmlNode nodeGeom, CFOPMasterInfo &masterInfo)
{
	if (false != nodeGeom.IsEmpty())
		return;

	FOPSimpleCompositePolygon curPath;

	CXDXmlNode childNodes = nodeGeom.GetChildNodes();
	int nSize = childNodes.GetNodeListSize();
	for (int i=0; i<nSize; i++)
	{
		CXDXmlNode childNode = childNodes.GetItemNode(i);
		CString strText = childNode.GetNodeName();

		if (strText == _T("MoveTo"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			curPath.MoveTo(FOPPoint(nX, nY));
		}
		else if (strText == _T("LineTo"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			curPath.LineTo(FOPPoint(nX, nY));
		}
		else if (strText == _T("ArcTo"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			double dA = childNode.GetNodeDouble(_T("A"));
			curPath.ArcTo(FOPPoint(nX, nY), dA);
		}
		else if (strText == _T("Ellipse"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			int nA = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("A")));
			int nB = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("B")));
			int nC = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("C")));
			int nD = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("D")));
			nX;nY;
			curPath.AddEllipse(FOPRect(nA, nB, nC, nD));
		}
		else if (strText == _T("InfiniteLine"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			int nA = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("A")));
			int nB = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("B")));
			curPath.AddLine(FOPPoint(nX, nY), FOPPoint(nA, nB));
		}
		else if (strText == _T("EllipticalArcTo"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			int nA = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("A")));
			int nB = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("B")));
			double dC = childNode.GetNodeDouble(_T("C"));
			double dD = childNode.GetNodeDouble(_T("D"));
			curPath.ArcTo(nX, nY, nA, nB, (int)dC, (int)dD);
		}
		else if (strText == _T("SplineStart"))
		{
		}
		else if (strText == _T("SplineKnot"))
		{
		}
		else if (strText == _T("PolylineTo"))
		{
			int nX = ConvertMM2Point_Horz(childNode.GetNodeDouble(_T("X")));
			int nY = ConvertMM2Point_Vert(childNode.GetNodeDouble(_T("Y")));
			double dA = childNode.GetNodeDouble(_T("A"));
//			curPath.PloyTo(FOPPoint(nX, nY), FOPPoint(nA, nB));
		}
		else if (strText == _T("NURBSTo"))
		{
		}
	}

	if (curPath.Count() > 0)
	{
		CFOPathShape *pReturn = new CFOPathShape;
		pReturn->AddRef();
		pReturn->Create(curPath);
		pReturn->SetEditPointAllowed(TRUE);

		pReturn->SetObjectCaption(masterInfo.strName);
		pReturn->SetToolTip(masterInfo.strPrompt);
		pReturn->SetShowLabelAtFirst(TRUE);

		m_pModel->InsertShape(pReturn);
		pReturn->Release();
		pReturn = NULL;
	}
}

float CFOPVisioStencilConvert::GetRotationAngle(double dAngle)
{
	return (float)(dAngle*(180.0/3.14159265358979));			
}

int CFOPVisioStencilConvert::ConvertMM2Point_Horz(double dHorz)
{
	m_pModel->ConvertToMapLog(FUNIT_MM, dHorz, FALSE);
	return int(dHorz);
}

int CFOPVisioStencilConvert::ConvertMM2Point_Vert(double dVert)
{
	m_pModel->ConvertToMapLog(FUNIT_MM, dVert, TRUE);
	return int(dVert);
}

void CFOPVisioStencilConvert::ProcessImageInfo(CXDXmlNode nodeImgInfo, ForeignImageInfo fii)
{
	if (false == nodeImgInfo.IsEmpty())
	{
		fii.m_nImageWidth = (int)ConvertToPixels(nodeImgInfo.GetNodeFloat(_T("ImgWidth")));
		fii.m_nImageHeight = (int)ConvertToPixels(nodeImgInfo.GetNodeFloat(_T("ImgHeight")));
		fii.m_fOffsetXToOrigin = ConvertToPixels(nodeImgInfo.GetNodeFloat(_T("ImgOffsetX")));
		fii.m_fOffsetYToOrigin = ConvertToPixels(nodeImgInfo.GetNodeFloat(_T("ImgOffsetY")));
	}
}

void CFOPVisioStencilConvert::ProcessXForm(CXDXmlNode nodeXForm, XForm &xfShapeProp)
{
	if (false == nodeXForm.IsEmpty())
	{
		xfShapeProp.m_fRotationAngle = nodeXForm.GetNodeFloat(_T("Angle"));
		xfShapeProp.m_fHeight = ConvertToPixels(nodeXForm.GetNodeFloat(_T("Height")));
		xfShapeProp.m_fWidth = ConvertToPixels(nodeXForm.GetNodeFloat(_T("Width")));
		xfShapeProp.m_bFlipX = (nodeXForm.GetNodeInt(_T("FlipX")) == 1);
		xfShapeProp.m_bFlipY = (nodeXForm.GetNodeInt(_T("FlipY")) == 1);
		xfShapeProp.m_fPinX = ConvertToPixels(nodeXForm.GetNodeFloat(_T("PinX")));
		xfShapeProp.m_fPinY = ConvertToPixels(nodeXForm.GetNodeFloat(_T("PinY")));
		xfShapeProp.m_fLocPinX = ConvertToPixels(nodeXForm.GetNodeFloat(_T("LocPinX")));
		xfShapeProp.m_fLocPinY = ConvertToPixels(nodeXForm.GetNodeFloat(_T("LocPinY")));
	}
}

void CFOPVisioStencilConvert::ProcessProtection(CXDXmlNode nodeProtection, EditStyle *pStlEdit)
{
	if (false == nodeProtection.IsEmpty())
	{
		pStlEdit->SetAllowChangeWidth((nodeProtection.GetNodeInt(_T("LockWidth")) == 0 ? true : false));
		pStlEdit->SetAllowChangeHeight((nodeProtection.GetNodeInt(_T("LockHeight")) == 0 ? true : false));
		pStlEdit->SetAllowMoveX((nodeProtection.GetNodeInt(_T("LockMoveX")) == 0 ? true : false));
		pStlEdit->SetAllowMoveY((nodeProtection.GetNodeInt(_T("LockMoveY")) == 0 ? true : false));
		pStlEdit->SetAspectRatio((nodeProtection.GetNodeInt(_T("LockAspect")) == 0 ? false : true));
		pStlEdit->SetAllowDelete((nodeProtection.GetNodeInt(_T("LockDelete")) == 0 ? true : false));
		pStlEdit->SetAllowRotate((nodeProtection.GetNodeInt(_T("LockRotate")) == 0 ? true : false));
		pStlEdit->SetAllowSelect((nodeProtection.GetNodeInt(_T("LockSelect")) == 0 ? true : false));
		pStlEdit->SetAllowVertexEdit((nodeProtection.GetNodeInt(_T("LockVtxEdit")) == 0 ? true : false));
	}
	else
	{
		_SafeDeletePtr(pStlEdit);
	}
}

// Text Processing Helpers

void CFOPVisioStencilConvert::ProcessText(CXDXmlNode nodeText, XForm &xfTextProp, XForm &xfShapeProp)
{
	xfTextProp;
	xfShapeProp;
	if (false == nodeText.IsEmpty())
	{
	}
}

bool CFOPVisioStencilConvert::CheckForNodeExistance(CXDXmlNode nodeToSearchIn, CString strNodeNameToSearchFor)
{
	if (false == nodeToSearchIn.IsEmpty())
	{
		CXDXmlNode node = nodeToSearchIn.GetNode(strNodeNameToSearchFor);
		if (false == node.IsEmpty())
			return true;
	}
	
	return false;
}

void CFOPVisioStencilConvert::ProcessTextXForm(CXDXmlNode nodeXForm, XForm &xfShapeProp)
{
	if (false == nodeXForm.IsEmpty())
	{
		xfShapeProp.m_fPinX = ConvertToPixels(nodeXForm.GetNodeFloat(_T("TxtPinX")));
		xfShapeProp.m_fPinY = ConvertToPixels(nodeXForm.GetNodeFloat(_T("TxtPinY")));
		xfShapeProp.m_fLocPinX = ConvertToPixels(nodeXForm.GetNodeFloat(_T("TxtLocpinX")));
		xfShapeProp.m_fLocPinY = ConvertToPixels(nodeXForm.GetNodeFloat(_T("TxtLocpinY")));
		xfShapeProp.m_fHeight = ConvertToPixels(nodeXForm.GetNodeFloat(_T("TxtHeight")));
		xfShapeProp.m_fWidth = ConvertToPixels(nodeXForm.GetNodeFloat(_T("TxtWidth")));
		xfShapeProp.m_fRotationAngle = nodeXForm.GetNodeFloat(_T("TxtAngle"));
	}
}

bool CFOPVisioStencilConvert::ShowGeom(CXDXmlNode nodeGeom)
{
	bool bSuccess = true;
	
	int nNoShow = SearchForSibling( nodeGeom, _T("NoShow"), _T(""), 0, true);
	if (nNoShow == 1)
			bSuccess = false;
	
	return bSuccess;
}

int CFOPVisioStencilConvert::SearchForSibling(CXDXmlNode nodeToSearch, CString strName, CString strAttributeName, int nAttrValue, bool bCaseSensitive)
{
	nAttrValue;
	bCaseSensitive;
	if (false == nodeToSearch.IsEmpty())
	{
		CXDXmlNode node = nodeToSearch.GetNode(strName);
		CXDNamedNode nodeName = node.GetNamedNode();
		return nodeName.GetNodeInt(strAttributeName);
	}

	return 0;
}

CXDXmlNode CFOPVisioStencilConvert::GetNodeByName( CString strNodeName, CXDXmlNode xmlNodeParent)
{
	return xmlNodeParent.GetNode(strNodeName);
}

bool CFOPVisioStencilConvert::HasBitmapOrMetafile(CXDXmlNode nodeFrgnData)
{
	CString strTemp = _T("");
	if (false == nodeFrgnData.IsEmpty())
	{
		CXDNamedNode nodeName = nodeFrgnData.GetNamedNode();
		strTemp = nodeName.GetNodeText(_T("ForeignType"));
	}
	
	return strTemp == c_strBITMAP || strTemp == c_strMETAFILE;
}

float CFOPVisioStencilConvert::GetRotationAngle (CXDXmlNode nodeAngle)
{
	float fRad = ProcessValue( nodeAngle, _T("Angle"));
	return (float)(fRad * ( 180.0 / 3.14159265358979323846));			
}

float CFOPVisioStencilConvert::ProcessValue(CXDXmlNode node, CString strItem)
{
	return node.GetNodeFloat(strItem);
}

COLORREF CFOPVisioStencilConvert::ProcessColor(CString strColor, bool bTextBlock)
{
	COLORREF clr = RGB(0, 0, 0);

	if (strColor.GetLength() == c_nHTML_RGB_LENGTH) // color hexadecimal representation
	{
	//	clr = ColorTranslator.FromHtml( strValue);
	}
	else // get color from m_lstColors 
	{
		int nIndex = _ttoi(strColor);
		// Need to subtract 1 from nIndex if we are in TextBlock
		if (bTextBlock)
		{
			if ( nIndex != 0)
			{
				nIndex--;
				m_hashColors.Lookup(nIndex, clr);
			}
		}
		else
		{
			clr = m_hashColors[nIndex];
		}
	}
	
	return clr;	
}

float CFOPVisioStencilConvert::ConvertToPixels(float fValue)
{	
	return fValue;
}

bool CFOPVisioStencilConvert::GetAttribute(CXDXmlNode node, int &nValue, CString strAttributeName)
{
	CXDNamedNode nodeName = node.GetNamedNode();

	double dRet = nodeName.GetNodeDouble(strAttributeName);
	nValue = int(dRet);

	return true;
}
